/* 
 * Copyright (C) 1999 Canek Pelez Valds <canek@abulafia.fciencias.unam.mx>
 * Copyright (C) 2007 Manuel Teira Paz <manuel.teira@telefonica.net>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef DEFS_H
#define DEFS_H

#ifdef __DEBUG__
# define F_NAME() g_print ("----- %s -----\n", G_GNUC_FUNCTION)
#else
# define F_NAME() ;
#endif

/* Default png. */
#define DEFAULT_IMAGE PIXMAPDIR "/gpe-snesdefault.png"
/* Configuration files directory. */
#define CONF_DIR      "gpe-snes"
/* ROMs titles file. */
#define ROMS_FILE     CONF_DIR "/ROMS"
/* Global preferences file. */
#define CONF_FILE     CONF_DIR "/gpe-snes.conf"

/* Max. length of each line in the configuration files. */
#define CONF_STR_LEN    512

/* 256x239 is the default size of Snes9X in X. */
#define IMAGE_WIDTH     256
#define IMAGE_HEIGHT    239

#define MIX_BOOL_PREFS   17
#define ROM_BOOL_PREFS   13
#define GLOB_BOOL_PREFS   0

#define MIX_STR_PREFS     0
#define ROM_STR_PREFS     5
#define GLOB_STR_PREFS    2

#define MIX_NUM_PREFS     6
#define ROM_NUM_PREFS     0
#define GLOB_NUM_PREFS    0

#define CODES             3

typedef struct _GsnesData      GsnesData;
typedef struct _GlobalPrefs      GlobalPrefs;
typedef struct _GlobalPrefsData  GlobalPrefsData;
typedef struct _ROMInfo          ROMInfo;
typedef struct _ROMPrefsData     ROMPrefsData;
typedef struct _BrowseStruct     BrowseStruct;
typedef struct _AddROMInfo       AddROMInfo;
typedef struct _CheatCode        CheatCode;
typedef struct _CodesListStruct  CodesListStruct;
typedef struct _AddCodeStruct    AddCodeStruct;

typedef enum {
    /* Mixed prefs. */
    TRANSPARENCY      =  0, /* Graphics options */
    SIXTEEN           =  1,
    HIRES             =  2,
    DISPLAYFRAMERATE  =  3,
    NOSOUND           =  4, /* Sound options. */
    SOUNDSKIP         =  5,
    SOUNDQUALITY      =  6,
    ENVX              =  7,
    SOUNDSYNC         =  8,
    INTERPOLATEDSOUND =  9,
    BUFFERSIZE        = 10,
    NOSAMPLECACHING   = 11,
    NOECHO            = 12,
    NOMASTERVOLUME    = 13, 
    FRAMESKIP         = 14,  /* Speed up/slow options. */
    FRAMETIME         = 15,
    CYCLES            = 16, 
    /* ROM only prefs. */
    INTERLEAVED       = 17, /* ROM image format options */
    INTERLEAVED2      = 18,
    HIROM             = 19,
    LOROM             = 20,
    HEADER            = 21,
    NOHEADER          = 22,
    PAL               = 23,
    NTSC              = 24,
    LAYERING          = 25,
    LOADSNAPSHOT      = 26,
    NOHDMA            = 27,
    NOSPEEDHACKS      = 28,
    NOWINDOWS         = 29,
} GsnesBoolPrefs;

typedef enum {
    /* Mix. prefs. */
    /* empty */
    /* ROM only prefs. */
    ROM_FILE       = 0,
    NAME           = 1,
    COMPANY        = 2,
    IMAGE_FILE     = 3,
    SNAPSHOT_FILE  = 4,
    /* Global only prefs. */
    ROMS_DIR       = 5,
    SNES9X_COMMAND = 6,
} GsnesStringPrefs;

typedef enum {
    SOUNDSKIP_NUMBER    = 0,
    SOUNDQUALITY_NUMBER = 1,
    BUFFERSIZE_NUMBER   = 2,
    FRAMESKIP_NUMBER    = 3,
    FRAMETIME_NUMBER    = 4,
    CYCLES_NUMBER       = 5,
} GsnesNumberPrefs;

typedef enum {
    BOOLEAN_KEYWORD = 0,
    NUMBER_KEYWORD  = 1,
    STRING_KEYWORD  = 2,
    UNKNOW_KEYWORD  = 3
} GsnesKeywordTypes;

/* General Preferences */
struct _GlobalPrefs {
    /* Bools. */
    gboolean   bool_prefs[MIX_BOOL_PREFS+GLOB_BOOL_PREFS];
    /* Strings. */
    gchar     *string_prefs[MIX_STR_PREFS+GLOB_STR_PREFS];
    /* Numbers. */
    gfloat     number_prefs[MIX_NUM_PREFS+GLOB_NUM_PREFS];
};

/* Each ROM*/
struct _ROMInfo {
    /* Bools. */
    gboolean  bool_prefs[MIX_BOOL_PREFS+ROM_BOOL_PREFS];
    /* Strings. */
    gchar    *string_prefs[MIX_STR_PREFS+ROM_STR_PREFS];
    /* Numbers. */
    gfloat    number_prefs[MIX_NUM_PREFS+ROM_NUM_PREFS];
};

struct _GsnesData {
    GtkWidget   *app;            /* Main Window.      */
    GtkWidget   *popmenu;        /* Pop Up menu.      */
    GList       *rom_widgets;    /* ROM sensible widgets. */
    GtkWidget   *paned;          /* ROM list/Image.   */
    GtkWidget   *image_w;        /* Image window.     */
    GtkWidget   *roms_w;         /* ROM list window.  */
    GtkWidget   *pixmap;         /* Image.            */
    GtkWidget   *clist;          /* ROM list.         */
    GtkWidget   *status;         /* Label of status.  */
    GlobalPrefs *prefs;          /* Global Prefs.     */
    GList       *roms_list;      /* List of ROMs info */
    gint         current_rom;    /* The selected ROM  */
    gint         width, height;  /* Width & length of the pixmap window. */
};

struct _BrowseStruct {
    GtkWidget *file_selection;
    GtkWidget *entry;
};

struct _AddROMInfo {
    GtkWidget   *entry[ROM_STR_PREFS-1];
    GtkWidget   *dialog;
    GsnesData *data;
};	

struct _GlobalPrefsData {
    gboolean   changed;
    gboolean   nodata;
	
    GtkWidget *global_prefs_dialog;

    /* ----- Check buttons -----. */
    GtkWidget *check[MIX_BOOL_PREFS+GLOB_BOOL_PREFS];
    /* ----- Text Entries ----- */
    GtkWidget *entry[GLOB_STR_PREFS];
    /* ----- Spin Buttons ----- */
    GtkWidget *spin[MIX_NUM_PREFS+GLOB_NUM_PREFS];
    /* ----- Buttons ----- */
    /* ----- Labels ----- */
    GsnesData *data;
};

struct _ROMPrefsData {
    gboolean   changed;
    gboolean   nodata;
    GtkWidget *rom_prefs_dialog;

    /* ----- Check buttons. ----- */
    GtkWidget *check[MIX_BOOL_PREFS+ROM_BOOL_PREFS];
    /* ----- Text Entries ----- */
    GtkWidget *entry[MIX_STR_PREFS+ROM_STR_PREFS];
    /* ----- Spin Buttons ----- */
    GtkWidget *spin[MIX_NUM_PREFS+ROM_NUM_PREFS];

    /* ----- Buttons ----- */
    GtkWidget *load_defaults;	
    GtkWidget *browse_snapfile;
    GsnesData  *data;
    ROMInfo      *rom_info;
};

#endif
