/* 
 * Copyright (C) 1999 Canek Pelez Valds <canek@abulafia.fciencias.unam.mx>
 * Copyright (C) 2007 Manuel Teira Paz <manuel.teira@telefonica.net>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gpe-snes.h>
#include <string.h>

static GlobalPrefsData* global_prefs_data_new();
static GtkWidget* global_prefs_system_ops(GlobalPrefsData *gpd);
static GtkWidget* global_prefs_graphics_ops(GlobalPrefsData *gpd);
static GtkWidget* global_prefs_sound_ops(GlobalPrefsData *gpd);
static GtkWidget* global_prefs_speed_ops(GlobalPrefsData *gpd);
static void global_prefs_nosound_cb(GtkWidget *widget, GlobalPrefsData *gpd);
static void global_prefs_dialog_changed_cb(GtkWidget *widget, 
                                           GlobalPrefsData *rpd);
static void global_prefs_connect_changes(GlobalPrefsData *gpd);
static void global_prefs_set_tooltips(GlobalPrefsData *gpd);
static void global_prefs_dialog_set_status(GlobalPrefsData *gpd);
static void global_prefs_dialog_get_status(GlobalPrefsData *rpd);
static void global_prefs_apply(GtkWidget *widget, GlobalPrefsData *gpd);
static void no_global_data(GlobalPrefsData *gpd);

void global_preferences_cb(GtkWidget *widget, GsnesData *gsnes_data)
{
    GlobalPrefsData *gpd = global_prefs_data_new();
    GtkWidget       *prefs_dialog;
    GtkWidget       *notebook;
    GtkWidget       *cont;

    gpd->data = gsnes_data;

    prefs_dialog = 
        gtk_dialog_new_with_buttons(_("GSNES Global Preferences"),
                                    GTK_WINDOW(gsnes_data->app),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                    NULL);
                                                   
    g_object_set_data(G_OBJECT(prefs_dialog), "global-prefs",
                      (gpointer) gpd);
    gpd->global_prefs_dialog = prefs_dialog;

    notebook = gtk_notebook_new();

    cont = global_prefs_system_ops(gpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("System")));
    gtk_widget_show(cont);
	
    cont = global_prefs_graphics_ops(gpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Graphics")));
    gtk_widget_show(cont);

    cont = global_prefs_sound_ops(gpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Sound")));
    gtk_widget_show(cont);
	
    cont = global_prefs_speed_ops(gpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Speed up/down")));
    gtk_widget_show(cont);

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox),
                       notebook, FALSE, FALSE, 10);
    gtk_widget_show(notebook);
	
    global_prefs_dialog_set_status(gpd);
    global_prefs_set_tooltips(gpd);
    global_prefs_connect_changes(gpd);

    gtk_dialog_run(GTK_DIALOG(prefs_dialog));
    global_prefs_apply(prefs_dialog, (gpointer) gpd);
    gtk_widget_destroy(prefs_dialog);
    g_free(gpd);
}

GlobalPrefs* global_prefs_new()
{
    gint i, j;
    GlobalPrefs *gp = g_new(GlobalPrefs, 1);

    for (i = 0; i < MIX_BOOL_PREFS + GLOB_BOOL_PREFS; i++) {
        gp->bool_prefs[i] = FALSE;
    }

    for (i = 0; i < GLOB_STR_PREFS; i++) {
        gp->string_prefs[i] = NULL;
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        gp->number_prefs[i] = 0;
    }
    return gp;
}

GlobalPrefsData* global_prefs_data_new()
{
    gint i, j;
    GlobalPrefsData *gp = g_new(GlobalPrefsData, 1);

    gp->changed = FALSE;
    gp->nodata  = FALSE;

    gp->global_prefs_dialog = NULL;

    for (i = 0; i < MIX_BOOL_PREFS+GLOB_BOOL_PREFS; i++) {
        gp->check[i] = NULL;
    }

    for (i = 0; i < MIX_STR_PREFS+GLOB_STR_PREFS; i++) {
        gp->entry[i] = NULL;
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        gp->spin[i] = NULL;
    }
    gp->data = NULL;
	
    return gp;
}

GtkWidget* global_prefs_system_ops(GlobalPrefsData *gpd)
{
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *text;
    GtkWidget *check;

    table = gtk_table_new(2, 2, FALSE);
	
    label = gtk_label_new(_("Snes9X command:"));
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);

    label = gtk_label_new(_("ROM directory:"));
    gtk_misc_set_alignment(GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE (table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);
	
    text = gtk_entry_new_with_max_length(CONF_STR_LEN);
    gtk_table_attach(GTK_TABLE (table), text, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    gpd->entry[SNES9X_COMMAND-ROM_STR_PREFS] = text;

    text = gtk_file_chooser_button_new(_("Select ROM directory"),
                                       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
        
    gtk_table_attach(GTK_TABLE(table), text, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    gpd->entry[ROMS_DIR-ROM_STR_PREFS] = text;

    return table;
}

GtkWidget* global_prefs_graphics_ops(GlobalPrefsData *gpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *frame;
    GtkWidget *option;
    GtkWidget *menu;
    GtkWidget *item;

    table = gtk_table_new(4, 2, TRUE);
	
    check = gtk_check_button_new_with_label(_("Transparency effects"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[TRANSPARENCY] = check;
	
    check = 
        gtk_check_button_new_with_label(_("16-bit internal screen rendering"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[SIXTEEN] = check;
	
    check = gtk_check_button_new_with_label(_("Hi-res"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[HIRES] = check;

    check = gtk_check_button_new_with_label(_("Display frame rate"));
    gtk_table_attach(GTK_TABLE (table), check, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[DISPLAYFRAMERATE] = check;

    return table;
}

GtkWidget* global_prefs_sound_ops(GlobalPrefsData *gpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *radio;
    GtkWidget *spin;
    GtkWidget *hbox;
    GtkObject *adj;
	
    table = gtk_table_new(4, 3, TRUE);

    check = gtk_check_button_new_with_label(_("No sound"));
    gtk_table_attach(GTK_TABLE (table), check, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[NOSOUND] = check;

    check = gtk_check_button_new_with_label(_("Volume envelope"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[ENVX] = check;

    check = gtk_check_button_new_with_label(_("Sound sync"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[SOUNDSYNC] = check;

    check = gtk_check_button_new_with_label(_("No sample caching"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[NOSAMPLECACHING] = check;

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE (table), hbox, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label(_("Sound skip"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show(check);
    gpd->check[SOUNDSKIP] = check;
	
    adj = gtk_adjustment_new(0, 0, 3, 1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show(spin);
    gpd->spin[SOUNDSKIP_NUMBER] = spin;

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label(_("Sound quality"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show(check);
    gpd->check[SOUNDQUALITY] = check;

    adj = gtk_adjustment_new(4, 0, 7, 1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show(spin);
    gpd->spin[SOUNDQUALITY_NUMBER] = spin;

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label(_("Buffer size"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show(check);
    gpd->check[BUFFERSIZE] = check;

    adj = gtk_adjustment_new(128, 128, 4096, 1, 8, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT (adj), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show(spin);
    gpd->spin[BUFFERSIZE_NUMBER] = spin;

    check = gtk_check_button_new_with_label(_("Interpolated sound"));
    gtk_table_attach(GTK_TABLE(table), check, 1, 2, 3, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[INTERPOLATEDSOUND] = check;


    check = gtk_check_button_new_with_label(_("No echo"));
    gtk_table_attach(GTK_TABLE (table), check, 2, 3, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[NOECHO] = check;
		
    check = gtk_check_button_new_with_label(_("No master volume"));
    gtk_table_attach(GTK_TABLE(table), check, 2, 3, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[NOMASTERVOLUME] = check;

    g_signal_connect(G_OBJECT(gpd->check[NOSOUND]), "toggled",
                     G_CALLBACK(global_prefs_nosound_cb),
                     (gpointer) gpd);
	  
    g_signal_connect(G_OBJECT(gpd->check[SOUNDSKIP]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (gpd->spin[SOUNDSKIP_NUMBER]));
	  
    g_signal_connect(G_OBJECT(gpd->check[SOUNDQUALITY]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (gpd->spin[SOUNDQUALITY_NUMBER]));
	
    g_signal_connect(G_OBJECT(gpd->check[BUFFERSIZE]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (gpd->spin[BUFFERSIZE_NUMBER]));
	
    return table;
}

GtkWidget* global_prefs_speed_ops(GlobalPrefsData *gpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *spin;
    GtkObject *adj;

    table = gtk_table_new(3, 2, TRUE);

    check = gtk_check_button_new_with_label(_("Frame skip"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[FRAMESKIP] = check;

    adj = gtk_adjustment_new(1, 1, G_MAXINT, 1, 10, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT (adj), 0.5, 0);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    gpd->spin[FRAMESKIP_NUMBER] = spin;

    check = gtk_check_button_new_with_label(_("Frame time"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[FRAMETIME] = check;

    adj = gtk_adjustment_new(1, 1, 100, 0.1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT (adj), 0.5, 1);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    gpd->spin[FRAMETIME_NUMBER] = spin;

    check = gtk_check_button_new_with_label(_("Cycles"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    gpd->check[CYCLES] = check;

    adj = gtk_adjustment_new(100, 0, 200, 1, 10, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT (adj), 0.5, 0);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    gpd->spin[CYCLES_NUMBER] = spin;

    g_signal_connect(G_OBJECT(gpd->check[FRAMESKIP]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) gpd->spin[FRAMESKIP_NUMBER]);
	  
    g_signal_connect(G_OBJECT(gpd->check[FRAMETIME]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) gpd->spin[FRAMETIME_NUMBER]);
	
    g_signal_connect(G_OBJECT(gpd->check[CYCLES]), "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) gpd->spin[CYCLES_NUMBER]);
	
    return table;
}

void global_prefs_nosound_cb(GtkWidget *widget, GlobalPrefsData *gpd)
{
    gint i;	

    if (GTK_TOGGLE_BUTTON(widget)->active) {
        gtk_widget_set_sensitive(gpd->spin[SOUNDSKIP_NUMBER],
                                 GTK_TOGGLE_BUTTON(gpd->check[SOUNDSKIP])->active);

        for (i = SOUNDQUALITY; i <= NOMASTERVOLUME; i++) {
            gtk_widget_set_sensitive(gpd->check[i], FALSE);
        }
		
        gtk_widget_set_sensitive(gpd->check[SOUNDSKIP], TRUE);
		
        gtk_widget_set_sensitive(gpd->spin[SOUNDQUALITY_NUMBER], FALSE);
        gtk_widget_set_sensitive(gpd->spin[BUFFERSIZE_NUMBER], FALSE);
    } else {
        gtk_widget_set_sensitive(gpd->spin[SOUNDSKIP_NUMBER], FALSE);

        for (i = SOUNDQUALITY; i <= NOMASTERVOLUME; i++) {
            gtk_widget_set_sensitive(gpd->check[i], TRUE);
        }

        gtk_widget_set_sensitive(gpd->check[SOUNDSKIP], FALSE);
		
        gtk_widget_set_sensitive(gpd->spin[SOUNDQUALITY_NUMBER],
                                 GTK_TOGGLE_BUTTON(gpd->check[SOUNDQUALITY])->active);
        gtk_widget_set_sensitive(gpd->spin[BUFFERSIZE_NUMBER],
                                 GTK_TOGGLE_BUTTON(gpd->check[BUFFERSIZE])->active);
    }	
}

void global_prefs_dialog_changed_cb(GtkWidget *widget, GlobalPrefsData *gpd)
{	
    gpd->nodata = FALSE;
    gpd->changed = TRUE;
}

void global_prefs_connect_changes(GlobalPrefsData *gpd)
{
    gint i, j;

    for (i = 0; i < GLOB_STR_PREFS; i++) {
        if (i == (ROMS_DIR-ROM_STR_PREFS)) {
            g_signal_connect(G_OBJECT(gpd->entry[i]), "selection-changed",
                             G_CALLBACK(global_prefs_dialog_changed_cb),
                             (gpointer) gpd);
        } else {
            g_signal_connect(G_OBJECT(gpd->entry[i]), "changed",
                             G_CALLBACK(global_prefs_dialog_changed_cb),
                             (gpointer) gpd);
        }
    }

    for (i = 0; i < MIX_BOOL_PREFS+GLOB_BOOL_PREFS; i++) {
        g_signal_connect(G_OBJECT(gpd->check[i]), "toggled",
                         G_CALLBACK(global_prefs_dialog_changed_cb),
                         (gpointer) gpd);
    }

    for (i = 0; i < MIX_NUM_PREFS+GLOB_NUM_PREFS; i++) {
        g_signal_connect(G_OBJECT(gpd->spin[i]), "changed",
                         G_CALLBACK(global_prefs_dialog_changed_cb),
                         (gpointer) gpd);
    }
}

void global_prefs_set_tooltips(GlobalPrefsData *gpd)
{
    gint i;
    GtkTooltips *tool;

    F_NAME();
	
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip(tool, gpd->check[TRANSPARENCY],
                         _("Enable transparency effects, also enables "
                           "16-bit screen mode selection."
                           "Transparency effects are automatically "
                           "enabled if the depth of your X  "
                           "server is 15 or greater."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[SIXTEEN],
                          _("Enable 16-bit internal screen rendering, "
                            "allows palette changes but no "
                            "transparency effects."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[HIRES],
                          _("Enable support for SNES hi-res. and "
                            "interlace modes. USE ONLY IF GAME "
                            "REQUIRES IT (FEW DO) BECAUSE IT REALLY "
                            "SLOWS DOWN THE EMULATOR."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[DISPLAYFRAMERATE],
                          _("Display a frame rate counter superimposed "
                            "over the bottom, left-hand corner "
                            "of the SNES display. The value before the "
                            "slash (/) indicates how many "
                            "frames per second are being rendered "
                            "compared to a real SNES, the value of "
                            "which is displayed after the slash."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip(tool, gpd->check[NOSOUND],
                         _("Disable sound CPU emulation and sound "
                           "output, useful for the few ROMs "
                           "where sound emulation causes them to lock "
                           "up due to timing errors."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[SOUNDSKIP],
                          _("Sound skip. ONLY USED IF SOUND IS DISABLED."),
                          NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[SOUNDQUALITY],
                          _("Sound playback rate/quality: "
                            "0 - disable sound, 1 - 8192, 2 - 11025, "
                            "3 - 16500, 4 - 22050 (default), "
                            "5 - 29300, 6 - 36600, 7 - 44000."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip(tool, gpd->check[ENVX],
                         _("Enable volume envelope height reading by "
                           "the sound CPU. Can cure sound "
                           "repeat problems with some games (e.g. "
                           "Mortal Kombat series), while causing "
                           "others to lock if enabled (eg. Bomberman "
                           "series). Use with the \'Sound sync\' "
                           "option to fix the problems this option can "
                           "otherwise create."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[SOUNDSYNC],
                          _("Turns on sound-sync code where sound data "
                            "is generated in sync with the "
                            "emulation of the SNES sound CPU "
                            "instructions. Slighty slower emulation "
                            "speed but helps reduce note and sound "
                            "effects timing problems and one or two  "
                            "games actually require this mode, notably "
                            "Williams Arcade Classics. "
                            "Also turns on interpolated sound and "
                            "envelope height reading. "
                            "Pressing the \']\' key during gameplay "
                            "toggles sound-sync only on and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[INTERPOLATEDSOUND],
                          _("Turns on interpolated sound. Any samples "
                            "played at a lower frequency than "
                            "the current playback rate (\'Sound quality\' "
                            "option) have additional sample points "
                            "computed and played where previously the "
                            "same sample data was played "
                            "repeatedly. Lower frequency sounds have "
                            "improved bass and sound much less "
                            "harsh - I suspect the real SNES does this. "
                            "Pressing \'[\' during gameplay toggles the "
                            "feature on and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[BUFFERSIZE],
                          _("Sound playback buffer size in bytes "
                            "128-4096."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[NOSAMPLECACHING],
                          _("Disable decompressed sound sample caching. "
                            "Decompressing samples takes time, "
                            "slowing down the emulator. Normally the "
                            "decompressed samples are saved "
                            "just in case they need to be played again, "
                            "but the way samples are stored "
                            "and played on the SNES, it can result in a "
                            "click sound or distortion when "
                            "caching samples with loops in them."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[NOECHO],
                          _("Turn off sound echo and FIR filter effects. "
                            "Processing these effects can "
                            "really slow down a non-MMX Pentium machine "
                            "due to the number of calculations "
                            "required to implement these features."),
                          NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[NOMASTERVOLUME],
                          _("Disable emulation of the sound DSP master "
                            "volume control. Some ROMs set "
                            "the volume level very low requiring you to "
                            "turn up the volume level of "
                            "your speakers introducing more background "
                            "noise. Use this option to "
                            "always have the master volume set on full "
                            "and to by-pass a bug which "
                            "prevents the music and sound effects being "
                            "heard on Turrican."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[FRAMESKIP],
                          _("Set this value to deliberately fix the "
                            "frame skip rate and disable auto-speed "
                            "regulation. Use a larger value faster "
                            "emulation but more jerky movement and a "
                            "smaller value for smooth but slower screen "
                            "updates. Use \'+\' and \'-\' keys to modify "
                            "the value during a game. Ideal for some "
                            "Super FX games that confuse the auto-adjust "
                            "code or for games that deliberately flash "
                            "the screen every alternate frame."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[FRAMETIME],
                          _("(default: 16.6ms NTSC games and 20ms for "
                            "PAL games) If auto-adjust frame skip option "
                            "is in effect, then the emulator will try "
                            "to maintain a constant game and music speed "
                            "locked to this value by skipping "
                            "the rendering of some frames or waiting "
                            "until the required time is reached. "
                            "Increase the value to slow down games, "
                            "decrease it to speed up games. "
                            "During a game the value can be adjusted "
                            "in millisecond steps by pressing "
                            "Shift \'-\' or Shift \'+\'."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, gpd->check[CYCLES],
                          _("Percentage of CPU cycles to execute per "
                            "scan line, decrease value to "
                            "increase emulation frame rate. Most "
                            "ROMs work with a value of 85 or above."),
                          NULL);
}

void
global_prefs_dialog_set_status(GlobalPrefsData *gpd)
{
    gint i, j;
    const gchar *path;
    gint numberops[] = {
        SOUNDSKIP,
        SOUNDQUALITY,
        BUFFERSIZE,
        FRAMESKIP,
        FRAMETIME,
        CYCLES
    };

    gtk_entry_set_text(GTK_ENTRY(gpd->entry[SNES9X_COMMAND-ROM_STR_PREFS]),
                       gpd->data->prefs->string_prefs[SNES9X_COMMAND-ROM_STR_PREFS]);
    path = gpd->data->prefs->string_prefs[ROMS_DIR-ROM_STR_PREFS];
    if (!path) {
        path = get_default_roms_dir();
    }
    if (path) {
        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(gpd->entry[ROMS_DIR-ROM_STR_PREFS]), path);
    }
    
    for (i = 0; i < MIX_BOOL_PREFS+GLOB_BOOL_PREFS; i++) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gpd->check[i]),
                                     gpd->data->prefs->bool_prefs[i]);
    }
	
    for (i = 0; i < MIX_NUM_PREFS; i++) {
        if (gpd->data->prefs->bool_prefs[(numberops[i])]) {
            gtk_spin_button_set_value(GTK_SPIN_BUTTON (gpd->spin[i]),
                                      gpd->data->prefs->number_prefs[i]);
        } else {
            gtk_widget_set_sensitive(gpd->spin[i], FALSE);
        }
    }
	
    if (gpd->data->prefs->bool_prefs[NOSOUND]) {
        for (i = SOUNDQUALITY; i < NOMASTERVOLUME; i++)
            gtk_widget_set_sensitive(gpd->check[i], FALSE);
    } else {
        gtk_widget_set_sensitive(gpd->check[SOUNDSKIP], FALSE);
        gtk_widget_set_sensitive(gpd->spin[SOUNDSKIP_NUMBER], FALSE);
    }
}

void global_prefs_dialog_get_status(GlobalPrefsData *gpd)
{
    const gchar *text;
    gint   i, j, number;
    
    text = 
        gtk_entry_get_text(GTK_ENTRY(gpd->entry[SNES9X_COMMAND-ROM_STR_PREFS]));
    g_free(gpd->data->prefs->string_prefs[SNES9X_COMMAND-ROM_STR_PREFS]);
    gpd->data->prefs->string_prefs[SNES9X_COMMAND-ROM_STR_PREFS] = 
        g_strdup(text);

    text = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(gpd->entry[ROMS_DIR-ROM_STR_PREFS]));
    g_free(gpd->data->prefs->string_prefs[ROMS_DIR-ROM_STR_PREFS]);
    gpd->data->prefs->string_prefs[ROMS_DIR-ROM_STR_PREFS] = g_strdup(text);
	
    for (i = 0; i < MIX_BOOL_PREFS+GLOB_BOOL_PREFS; i++) {
        gpd->data->prefs->bool_prefs[i] =
            GTK_TOGGLE_BUTTON(gpd->check[i])->active;
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        gpd->data->prefs->number_prefs[i] =
            gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(gpd->spin[i]));
    }
}

void global_prefs_apply(GtkWidget *widget, GlobalPrefsData *gpd)
{
    gchar *text;
    if (!gpd->changed) {
        return;
    }
    
    text = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(gpd->entry[ROMS_DIR-ROM_STR_PREFS]));
    if (!text || !strlen(text)) {
	gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(gpd->entry[ROMS_DIR-ROM_STR_PREFS]),
                                 get_default_roms_dir());
    }
		
    if (!strlen(gtk_entry_get_text(GTK_ENTRY(gpd->entry[SNES9X_COMMAND-ROM_STR_PREFS])))) {
        gpd->nodata = TRUE;
        no_global_data(gpd);
        return;
    }

    global_prefs_dialog_get_status(gpd);
    save_global_prefs(gpd->data->prefs);
    gpd->changed = FALSE;
}

gboolean global_prefs_close(GtkWidget *widget, GlobalPrefsData *gpd)
{
    GtkWidget *warning;
    gboolean retcode = FALSE;

    if (gpd->changed && !gpd->nodata) {
        warning = 
            gtk_message_dialog_new(GTK_WINDOW(gpd->global_prefs_dialog),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_WARNING,
                                   GTK_BUTTONS_YES_NO,
                                   _("You changed the global preferences.\n"
                                     "Are you sure?"));
        gint response = gtk_dialog_run(GTK_DIALOG(warning));
        switch (response) {
        case GTK_RESPONSE_YES:
            retcode = FALSE;
            break;
        default:
            retcode = TRUE;
            break;
        }
        gtk_widget_destroy(warning);
    }
    return retcode;
}

void no_global_data(GlobalPrefsData *gpd)
{
    GtkWidget *error;
    gchar     *mesg;

    mesg = g_strdup(_("You must fill the \"Snes9X command\" "
                      "entry."));
    
    error = gtk_message_dialog_new(GTK_WINDOW(gpd->global_prefs_dialog),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
                                   mesg);
    gtk_dialog_run(GTK_DIALOG(error));
    gtk_widget_destroy(error);
}

/* globalprefs.c ends here. */
