/* 
 * Copyright (C) 1999 Canek Pelez Valds <canek@abulafia.fciencias.unam.mx>
 * Copyright (C) 2007 Manuel Teira Paz <manuel.teira@telefonica.net>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpe-snes.h"
#include <string.h>

static ROMPrefsData* rom_prefs_data_new();
static GtkWidget* rom_prefs_info_ops(ROMPrefsData *rpd);
static GtkWidget* rom_prefs_graphics_ops(ROMPrefsData *rpd);
static GtkWidget* rom_prefs_sound_ops(ROMPrefsData *rpd);
static GtkWidget* rom_prefs_speed_ops(ROMPrefsData *rpd);
static GtkWidget* rom_prefs_format_ops(ROMPrefsData *rpd);
static void rom_prefs_nosound_cb(GtkWidget *widget, ROMPrefsData *rpd);
static void rom_prefs_dialog_changed_cb(GtkWidget *widget, ROMPrefsData *rpd);
static void rom_prefs_connect_changes(ROMPrefsData *rpd);
static void rom_prefs_set_tooltips(ROMPrefsData *rpd);
static void rom_prefs_dialog_set_status(ROMPrefsData *rpd);
static void rom_prefs_dialog_get_status(ROMPrefsData *rpd);

/* Dialog buttons. */
void rom_prefs_apply(GtkWidget *widget, ROMPrefsData *rpd);
gboolean rom_prefs_close(GtkWidget *widget, ROMPrefsData *rpd);
void rom_load_defaults_cb(GtkWidget *widget, ROMPrefsData *rpd);



void rom_preferences_cb(GtkWidget *widget, 
                        GsnesData *gsnes_data)
{
    ROMPrefsData *rpd = rom_prefs_data_new();
    GtkWidget *prefs_dialog;
    GtkWidget *notebook;
    GtkWidget *cont;
    GList *list;
    gchar *title;
    gchar *rom_title;

    if (gsnes_data->current_rom == -1) {
        g_free(rpd);
        return;
    }

    rpd->data = gsnes_data;
	
    gtk_clist_get_text(GTK_CLIST(gsnes_data->clist),
                       gsnes_data->current_rom, 0, &rom_title);

    list = g_list_first(gsnes_data->roms_list);

    while (strcmp(rom_title, ((ROMInfo*)(list->data))->string_prefs[NAME])) {
        list = g_list_next(list);
    }

    rpd->rom_info = (ROMInfo*)(list->data);
    title = g_strconcat(_("Preferences for "), rom_title, NULL);

    prefs_dialog = 
        gtk_dialog_new_with_buttons(title,
                                    GTK_WINDOW(gsnes_data->app),
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                    NULL);

    g_free(title);

    g_object_set_data(G_OBJECT(prefs_dialog), "rom-prefs", (gpointer) rpd);
	
    rpd->rom_prefs_dialog = prefs_dialog;

    notebook = gtk_notebook_new();

    cont = rom_prefs_info_ops(rpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("ROM Info")));
    gtk_widget_show(cont);
	
    cont = rom_prefs_graphics_ops(rpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Graphics")));
    gtk_widget_show(cont);

    cont = rom_prefs_sound_ops(rpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Sound")));
    gtk_widget_show(cont);

    cont = rom_prefs_speed_ops(rpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("Speed up/down")));
    gtk_widget_show(cont);

    cont = rom_prefs_format_ops(rpd);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cont,
                             gtk_label_new(_("ROM image format")));
    gtk_widget_show(cont);

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox),
                       notebook, FALSE, FALSE, 10);
    gtk_widget_show(notebook);
	
    rom_prefs_dialog_set_status(rpd);
    rom_prefs_connect_changes(rpd);
    rom_prefs_set_tooltips(rpd);

    gtk_dialog_run(GTK_DIALOG(prefs_dialog));
    rom_prefs_apply(prefs_dialog, rpd);
    gtk_widget_destroy(prefs_dialog);
    g_free(rpd);
}

ROMInfo *rom_info_new()
{
    gint i, j;
    ROMInfo *rom_info = g_new(ROMInfo, 1);

    for (i = 0; i < ROM_STR_PREFS; i++) {
        rom_info->string_prefs[i] = NULL;
    }
	
    for (i = 0; i < MIX_BOOL_PREFS+ROM_BOOL_PREFS; i++) {
        rom_info->bool_prefs[i] = 0;
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        rom_info->number_prefs[i] = 0;
    }
    return rom_info;
}

ROMPrefsData* rom_prefs_data_new()
{
	gint i, j;
	ROMPrefsData *rpd = g_new(ROMPrefsData, 1);	

	F_NAME();
	
	rpd->changed = FALSE;
	rpd->nodata = FALSE;

	for (i = 0; i < MIX_BOOL_PREFS+ROM_BOOL_PREFS; i++) {
            rpd->check[i] = NULL;
        }

	for (i = 0; i < MIX_NUM_PREFS; i++) {
            rpd->entry[i] = NULL;
        }
	
	for (i = 0; i < MIX_NUM_PREFS; i++) {
            rpd->spin[i] = NULL;
        }

	rpd->data     = NULL;
	rpd->rom_info = NULL;
	
	return rpd;
}

GtkWidget* rom_prefs_info_ops(ROMPrefsData *rpd)
{
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *text;
    GtkWidget *separator;
    GtkWidget *button;
    const gchar *path;
    const gchar *imagepath;

    table = gtk_table_new(4, 3, FALSE);

    label = gtk_label_new(_("ROM file:"));
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);

    text = gtk_file_chooser_button_new(_("Select ROM file"),
                                       GTK_FILE_CHOOSER_ACTION_OPEN);
    path = get_default_roms_dir();
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(text), path);

    gtk_table_attach(GTK_TABLE(table), text, 1, 2, 0, 1,
                     GTK_EXPAND|GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    rpd->entry[ROM_FILE] = text;
	
    label = gtk_label_new(_("Name:"));
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);

    text = gtk_entry_new_with_max_length(CONF_STR_LEN);
    gtk_table_attach(GTK_TABLE(table), text, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    rpd->entry[NAME] = text;

    label = gtk_label_new(_("Company:"));
    gtk_misc_set_alignment(GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE (table), label, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);

    text = gtk_entry_new_with_max_length(CONF_STR_LEN);
    gtk_table_attach(GTK_TABLE (table), text, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    rpd->entry[COMPANY] = text;

    label = gtk_label_new(_("Image File:"));
    gtk_misc_set_alignment(GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach(GTK_TABLE (table), label, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(label);

    text = gtk_file_chooser_button_new(_("Select Image file"),
                                       GTK_FILE_CHOOSER_ACTION_OPEN);
    imagepath = get_default_images_dir();
    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(text), imagepath);
    gtk_table_attach (GTK_TABLE(table), text, 1, 2, 3, 4,
                      GTK_EXPAND|GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(text);
    rpd->entry[IMAGE_FILE] = text;

    separator = gtk_vseparator_new();
    gtk_table_attach(GTK_TABLE(table), separator, 2, 3, 0, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(separator);

    button = gtk_button_new_with_label(_("Load Defaults"));
    gtk_table_attach(GTK_TABLE(table), button, 3, 4, 1, 2,
                     GTK_SHRINK, GTK_SHRINK, 5, 2);
    gtk_widget_show(button);
    rpd->load_defaults = button;
    g_signal_connect(G_OBJECT(rpd->load_defaults), "clicked",
                     G_CALLBACK(rom_load_defaults_cb),
                     (gpointer)(rpd));
    return table;
}

GtkWidget* rom_prefs_graphics_ops(ROMPrefsData *rpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *frame;
    GtkWidget *option;
    GtkWidget *menu;
    GtkWidget *item;
	
    table = gtk_table_new (4, 1, TRUE);
	
    check = gtk_check_button_new_with_label (_("Transparency effects"));
    gtk_table_attach (GTK_TABLE (table), check, 0, 1, 0, 1,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[TRANSPARENCY] = check;
	
    check = gtk_check_button_new_with_label
        (_("16-bit internal screen rendering"));
    gtk_table_attach (GTK_TABLE (table), check, 0, 1, 1, 2,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[SIXTEEN] = check;
	
    check = gtk_check_button_new_with_label (_("Hi-res"));
    gtk_table_attach (GTK_TABLE (table), check, 0, 1, 2, 3,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[HIRES] = check;

    check = gtk_check_button_new_with_label (_("Display frame rate"));
    gtk_table_attach (GTK_TABLE (table), check, 0, 1, 3, 4,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[DISPLAYFRAMERATE] = check;

    return table;
}

GtkWidget* rom_prefs_sound_ops(ROMPrefsData *rpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *radio;
    GtkWidget *spin;
    GtkObject *adj;	
    GtkWidget *hbox;
	
    table = gtk_table_new (4, 3, TRUE);

    check = gtk_check_button_new_with_label (_("No sound"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[NOSOUND] = check;

    check = gtk_check_button_new_with_label(_("Volume envelope"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[ENVX] = check;

    check = gtk_check_button_new_with_label(_("Sound sync"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[SOUNDSYNC] = check;

    check = gtk_check_button_new_with_label(_("No sample caching"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[NOSAMPLECACHING] = check;
	
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE (table), hbox, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label(_("Sound skip"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show (check);
    rpd->check[SOUNDSKIP] = check;
	
    adj = gtk_adjustment_new(0, 0, 3, 1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show (spin);
    rpd->spin[SOUNDSKIP_NUMBER] = spin;

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label (_("Sound quality"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show (check);
    rpd->check[SOUNDQUALITY] = check;

    adj = gtk_adjustment_new(4, 0, 7, 1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show (spin);
    rpd->spin[SOUNDQUALITY_NUMBER] = spin;

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    check = gtk_check_button_new_with_label (_("Buffer size"));
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    gtk_widget_show (check);
    rpd->check[BUFFERSIZE] = check;

    adj = gtk_adjustment_new (128, 128, 4096, 1, 8, 0);
    spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj),
                                0.5, 0);
    gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
    gtk_widget_show (spin);
    rpd->spin[BUFFERSIZE_NUMBER] = spin;

    check = gtk_check_button_new_with_label (_("Interpolated sound"));
    gtk_table_attach (GTK_TABLE (table), check, 1, 2, 3, 4,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[INTERPOLATEDSOUND] = check;



    check = gtk_check_button_new_with_label (_("No echo"));
    gtk_table_attach (GTK_TABLE (table), check, 2, 3, 0, 1,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[NOECHO] = check;
		
    check = gtk_check_button_new_with_label (_("No master volume"));
    gtk_table_attach (GTK_TABLE (table), check, 2, 3, 1, 2,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show (check);
    rpd->check[NOMASTERVOLUME] = check;

    g_signal_connect(G_OBJECT(rpd->check[NOSOUND]),
                     "toggled",
                     G_CALLBACK(rom_prefs_nosound_cb),
                     (gpointer) rpd);
	
    g_signal_connect(G_OBJECT(rpd->check[SOUNDSKIP]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (rpd->spin[SOUNDSKIP_NUMBER]));

    g_signal_connect(G_OBJECT(rpd->check[SOUNDQUALITY]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (rpd->spin[SOUNDQUALITY_NUMBER]));
	
    g_signal_connect(G_OBJECT(rpd->check[BUFFERSIZE]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) (rpd->spin[BUFFERSIZE_NUMBER]));
    
    return table;
}

GtkWidget* rom_prefs_speed_ops(ROMPrefsData *rpd)
{
    GtkWidget *table;
    GtkWidget *check;
    GtkWidget *spin;
    GtkObject *adj;	

    table = gtk_table_new(3, 2, TRUE);

    check = gtk_check_button_new_with_label(_("Frame skip"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[FRAMESKIP] = check;

    adj = gtk_adjustment_new(1, 1, G_MAXINT, 1, 10, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 0);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    rpd->spin[FRAMESKIP_NUMBER] = spin;


    check = gtk_check_button_new_with_label(_("Frame time"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[FRAMETIME] = check;

    adj = gtk_adjustment_new(1, 1, 100, 0.1, 1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.5, 1);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 1, 2,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    rpd->spin[FRAMETIME_NUMBER] = spin;

    check = gtk_check_button_new_with_label(_("Cycles"));
    gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                      GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(check);
    rpd->check[CYCLES] = check;

    adj = gtk_adjustment_new(100, 0, 200, 1, 10, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT (adj), 0.5, 0);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 5, 2);
    gtk_widget_show(spin);
    rpd->spin[CYCLES_NUMBER] = spin;

    g_signal_connect(G_OBJECT(rpd->check[FRAMESKIP]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) rpd->spin[FRAMESKIP_NUMBER]);
    
    g_signal_connect(G_OBJECT(rpd->check[FRAMETIME]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) rpd->spin[FRAMETIME_NUMBER]);

    g_signal_connect(G_OBJECT(rpd->check[CYCLES]),
                     "toggled",
                     G_CALLBACK(toggle_to_active_cb),
                     (gpointer) rpd->spin[CYCLES_NUMBER]);
	
    return table;
}

GtkWidget* rom_prefs_format_ops(ROMPrefsData *rpd)
{
	GtkWidget *table;
	GtkWidget *check;
	GtkWidget *text;

	F_NAME();
	
	table = gtk_table_new (4, 4, FALSE);

	check = gtk_check_button_new_with_label (_("Interleaved"));
	gtk_table_attach (GTK_TABLE (table), check, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[INTERLEAVED] = check;

	check = gtk_check_button_new_with_label (_("Interleaved 2"));
	gtk_table_attach (GTK_TABLE (table), check, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[INTERLEAVED2] = check;

	check = gtk_check_button_new_with_label (_("Hi-ROM"));
	gtk_table_attach (GTK_TABLE (table), check, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[HIROM] = check;

	check = gtk_check_button_new_with_label (_("Lo-ROM"));
	gtk_table_attach (GTK_TABLE (table), check, 0, 1, 3, 4,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[LOROM] = check;
	
	check = gtk_check_button_new_with_label (_("Header"));
	gtk_table_attach (GTK_TABLE (table), check, 1, 2, 0, 1,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[HEADER] = check;

	check = gtk_check_button_new_with_label (_("No header"));
	gtk_table_attach (GTK_TABLE (table), check, 1, 2, 1, 2,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[NOHEADER] = check;

	check = gtk_check_button_new_with_label (_("Pal"));
	gtk_table_attach (GTK_TABLE (table), check, 1, 2, 2, 3,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[PAL] = check;

	check = gtk_check_button_new_with_label (_("NTSC"));
	gtk_table_attach (GTK_TABLE (table), check, 1, 2, 3, 4,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[NTSC] = check;

	check = gtk_check_button_new_with_label (_("Layering"));
	gtk_table_attach (GTK_TABLE (table), check, 2, 3, 0, 1,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[LAYERING] = check;

	check = gtk_check_button_new_with_label (_("No H-DMA"));
	gtk_table_attach (GTK_TABLE (table), check, 2, 3, 1, 2,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[NOHDMA] = check;

	check = gtk_check_button_new_with_label (_("No speed hacks"));
	gtk_table_attach (GTK_TABLE (table), check, 2, 3, 2, 3,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[NOSPEEDHACKS] = check;

	check = gtk_check_button_new_with_label (_("No windows"));
	gtk_table_attach (GTK_TABLE (table), check, 2, 3, 3, 4,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[NOWINDOWS] = check;

	check = gtk_check_button_new_with_label (_("Load snapshot file"));
	gtk_table_attach (GTK_TABLE (table), check, 3, 4, 0, 1,
			  GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show (check);
	rpd->check[LOADSNAPSHOT] = check;

        text = gtk_file_chooser_button_new(_("Select Snapshot file"),
                                           GTK_FILE_CHOOSER_ACTION_OPEN);
	rpd->browse_snapfile = text;
        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(text),
                                            get_default_snap_dir());

	gtk_table_attach(GTK_TABLE (table), text, 3, 4, 1, 2,
                         GTK_FILL, GTK_FILL, 5, 2);
	gtk_widget_show(text);
        rpd->entry[SNAPSHOT_FILE] = text;
	
	g_signal_connect(G_OBJECT(rpd->check[LOADSNAPSHOT]), "toggled",
                         G_CALLBACK(toggle_to_active_cb),
                         (gpointer) rpd->entry[SNAPSHOT_FILE]);

	g_signal_connect(G_OBJECT(rpd->check[LOADSNAPSHOT]), "toggled",
                         G_CALLBACK(toggle_to_active_cb),
                         (gpointer) rpd->browse_snapfile);
	
	return table;
}

void rom_prefs_nosound_cb(GtkWidget *widget, ROMPrefsData *rpd)
{
    gint i;	

    if (GTK_TOGGLE_BUTTON (widget)->active) {
        gtk_widget_set_sensitive(rpd->spin[SOUNDSKIP_NUMBER],
                                 GTK_TOGGLE_BUTTON(rpd->check[SOUNDSKIP])->active);

        for (i = SOUNDQUALITY; i <= NOMASTERVOLUME; i++)
            gtk_widget_set_sensitive (rpd->check[i], FALSE);

        gtk_widget_set_sensitive(rpd->check[SOUNDSKIP], TRUE);
		
        gtk_widget_set_sensitive(rpd->spin[SOUNDQUALITY_NUMBER], FALSE);
        gtk_widget_set_sensitive(rpd->spin[BUFFERSIZE_NUMBER], FALSE);
    } else {
        gtk_widget_set_sensitive(rpd->spin[SOUNDSKIP_NUMBER], FALSE);

        for (i = SOUNDQUALITY; i <= NOMASTERVOLUME; i++) {
            gtk_widget_set_sensitive (rpd->check[i], TRUE);
        }

        gtk_widget_set_sensitive(rpd->check[SOUNDSKIP], FALSE);
		
        gtk_widget_set_sensitive(rpd->spin[SOUNDQUALITY_NUMBER],
                                 GTK_TOGGLE_BUTTON(rpd->check[SOUNDQUALITY])->active);
        gtk_widget_set_sensitive(rpd->spin[BUFFERSIZE_NUMBER],
                                 GTK_TOGGLE_BUTTON(rpd->check[BUFFERSIZE])->active);
    }
}

void rom_prefs_dialog_changed_cb(GtkWidget *widget, ROMPrefsData *rpd)
{
    FILE  *file;
    GList *list;
    gchar *rom_info_file, *name;
    gint   i, l;

    rpd->nodata = FALSE;
	
    if (rpd->changed)
        return;
	
    rom_info_file = g_strconcat(CONF_DIR, "/",
                                rpd->rom_info->string_prefs[NAME], NULL);
		
    remove(resolve_dir(rom_info_file));
	
    g_free(rom_info_file);

    list = g_list_first(rpd->data->roms_list);
    l    = g_list_length(list);

    if (l > 1) {
        file = fopen(resolve_dir(ROMS_FILE), "w");
		
        for (i = 0; i < l; i++) {
            name = ((ROMInfo*)
                    (list->data))->string_prefs[NAME];
            if (strcmp (name,
                        rpd->rom_info->string_prefs[NAME]))
                fprintf (file, "%s\n", name);
            list = g_list_next (list);
        }
        fclose (file);
    } else {
        remove(resolve_dir(ROMS_FILE));
    }

    rpd->changed = TRUE;
}

void rom_prefs_connect_changes(ROMPrefsData *rpd)
{
    gint i, j;

    for (i = 0; i < MIX_BOOL_PREFS+ROM_BOOL_PREFS; i++) {
        g_signal_connect(G_OBJECT(rpd->check[i]), "toggled",
			 G_CALLBACK(rom_prefs_dialog_changed_cb),
			 (gpointer) rpd);
    }
	
    for (i = 0; i < MIX_NUM_PREFS+ROM_NUM_PREFS; i++) {
        g_signal_connect(G_OBJECT(rpd->spin[i]), "changed",
                         G_CALLBACK(rom_prefs_dialog_changed_cb),
                         (gpointer) rpd);
    }
		
    for (i = 0; i < ROM_STR_PREFS; i++) {
        if ((i == ROM_FILE) ||
            (i == IMAGE_FILE) ||
            (i == SNAPSHOT_FILE)) {
            g_signal_connect(G_OBJECT(rpd->entry[i]), "selection-changed",
                             G_CALLBACK(rom_prefs_dialog_changed_cb),
                             (gpointer) rpd);
        } else {
            g_signal_connect(G_OBJECT(rpd->entry[i]), "changed",
                             G_CALLBACK(rom_prefs_dialog_changed_cb),
                             (gpointer) rpd);
        }
    }

    g_signal_connect(G_OBJECT(rpd->entry[SNAPSHOT_FILE]), "selection-changed",
                     G_CALLBACK(rom_prefs_dialog_changed_cb),
                     (gpointer) rpd);
}

void rom_prefs_set_tooltips(ROMPrefsData *rpd)
{
    gint         i;
    GtkTooltips *tool;

    F_NAME();
	
    tool = gtk_tooltips_new ();
	
    gtk_tooltips_set_tip (tool, rpd->check[TRANSPARENCY],
                          _("Enable transparency effects, also enables "
                            "16-bit screen mode selection."
                            "Transparency effects are automatically "
                            "enabled if the depth of your X  "
                            "server is 15 or greater."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[SIXTEEN],
                          _("Enable 16-bit internal screen rendering, "
                            "allows palette changes but no "
                            "transparency effects."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[HIRES],
                          _("Enable support for SNES hi-res. and "
                            "interlace modes. USE ONLY IF GAME "
                            "REQUIRES IT (FEW DO) BECAUSE IT REALLY "
                            "SLOWS DOWN THE EMULATOR."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[DISPLAYFRAMERATE],
                          _("Display a frame rate counter superimposed "
                            "over the bottom, left-hand corner "
                            "of the SNES display. The value before the "
                            "slash (/) indicates how many "
                            "frames per second are being rendered "
                            "compared to a real SNES, the value of "
                            "which is displayed after the slash."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOSOUND],
                          _("Disable sound CPU emulation and sound "
                            "output, useful for the few ROMs "
                            "where sound emulation causes them to lock "
                            "up due to timing errors."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[SOUNDSKIP],
                          _("Sound skip. ONLY USED IF SOUND IS DISABLED."),
                          NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[SOUNDQUALITY],
                          _("Sound playback rate/quality: "
                            "0 - disable sound, 1 - 8192, 2 - 11025, "
                            "3 - 16500, 4 - 22050 (default), "
                            "5 - 29300, 6 - 36600, 7 - 44000."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[ENVX],
                          _("Enable volume envelope height reading by "
                            "the sound CPU. Can cure sound "
                            "repeat problems with some games (e.g. "
                            "Mortal Kombat series), while causing "
                            "others to lock if enabled (eg. Bomberman "
                            "series). Use with the \'Sound sync\' "
                            "option to fix the problems this option can "
                            "otherwise create."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[SOUNDSYNC],
                          _("Turns on sound-sync code where sound data "
                            "is generated in sync with the "
                            "emulation of the SNES sound CPU "
                            "instructions. Slighty slower emulation "
                            "speed but helps reduce note and sound "
                            "effects timing problems and one or two  "
                            "games actually require this mode, notably "
                            "Williams Arcade Classics. "
                            "Also turns on interpolated sound and "
                            "envelope height reading. "
                            "Pressing the \']\' key during gameplay "
                            "toggles sound-sync only on and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[INTERPOLATEDSOUND],
                          _("Turns on interpolated sound. Any samples "
                            "played at a lower frequency than "
                            "the current playback rate (\'Sound quality\' "
                            "option) have additional sample points "
                            "computed and played where previously the "
                            "same sample data was played "
                            "repeatedly. Lower frequency sounds have "
                            "improved bass and sound much less "
                            "harsh - I suspect the real SNES does this. "
                            "Pressing \'[\' during gameplay toggles the "
                            "feature on and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[BUFFERSIZE],
                          _("Sound playback buffer size in bytes "
                            "128-4096."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOSAMPLECACHING],
                          _("Disable decompressed sound sample caching. "
                            "Decompressing samples takes time, "
                            "slowing down the emulator. Normally the "
                            "decompressed samples are saved "
                            "just in case they need to be played again, "
                            "but the way samples are stored "
                            "and played on the SNES, it can result in a "
                            "click sound or distortion when "
                            "caching samples with loops in them."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOECHO],
                          _("Turn off sound echo and FIR filter effects. "
                            "Processing these effects can "
                            "really slow down a non-MMX Pentium machine "
                            "due to the number of calculations "
                            "required to implement these features."),
                          NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOMASTERVOLUME],
                          _("Disable emulation of the sound DSP master "
                            "volume control. Some ROMs set "
                            "the volume level very low requiring you to "
                            "turn up the volume level of "
                            "your speakers introducing more background "
                            "noise. Use this option to "
                            "always have the master volume set on full "
                            "and to by-pass a bug which "
                            "prevents the music and sound effects being "
                            "heard on Turrican."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[FRAMESKIP],
                          _("Set this value to deliberately fix the "
                            "frame skip rate and disable auto-speed "
                            "regulation. Use a larger value faster "
                            "emulation but more jerky movement and a "
                            "smaller value for smooth but slower screen "
                            "updates. Use \'+\' and \'-\' keys to modify "
                            "the value during a game. Ideal for some "
                            "Super FX games that confuse the auto-adjust "
                            "code or for games that deliberately flash "
                            "the screen every alternate frame."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[FRAMETIME],
                          _("(default: 16.6ms NTSC games and 20ms for "
                            "PAL games) If auto-adjust frame skip option "
                            "is in effect, then the emulator will try "
                            "to maintain a constant game and music speed "
                            "locked to this value by skipping "
                            "the rendering of some frames or waiting "
                            "until the required time is reached. "
                            "Increase the value to slow down games, "
                            "decrease it to speed up games. "
                            "During a game the value can be adjusted "
                            "in millisecond steps by pressing "
                            "Shift \'-\' or Shift \'+\'."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[CYCLES],
                          _("Percentage of CPU cycles to execute per "
                            "scan line, decrease value to "
                            "increase emulation frame rate. Most "
                            "ROMs work with a value of 85 or above."),
                          NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[INTERLEAVED],
                          _("Force interleaved ROM image format."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[INTERLEAVED2],
                          _("Force alternate interleaved format (i.e. "
                            "most Super FX games)."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[HIROM],
                          _("Force Hi-ROM memory map for ROMs where the "
                            "Hi-ROM header test fails."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[LOROM],
                          _("Force Lo-ROM memory map for ROMs where the "
                            "Hi-ROM header test fails."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[HEADER],
                          _("Force the detection of a ROM image header. "
                            "Some ROM images have been  "
                            "hand-edited to remove unused space from the "
                            "end of the file; if the "
                            "resultant image size is not a multiple of "
                            "32k then Snes9X can\'t  "
                            "auto-detect the presense of a 512 byte ROM "
                            "image header."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOHEADER],
                          _("Force Snes9X into thinking no ROM image "
                            "header is present. See \'Header\' "
                            "above."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[PAL],
                          _("Fool ROM into thinking this is a PAL SNES "
                            "system and adjust frame time "
                            "to 20ms (50 frames per second) "), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NTSC],
                          _("Fool ROM into thinking this is a NTSC SNES "
                            "system and adjust frame time "
                            "to 16.7ms (60 frames per second) "), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[LAYERING],
                          _("Swap background layer priorities from "
                            "background involved in sub-screen "
                            "addition/subtraction. Can improve some "
                            "games play-ability - no need to "
                            "constantly toggle background layers on and "
                            "off to read text/see maps, etc. "
                            "Toggle feature on and off during game by "
                            "pressing \'8\'. "
                            "Not used if transparency effects are "
                            "enabled."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[LOADSNAPSHOT],
                          _("Load snapshot file and restart game from "
                            "saved position."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOHDMA],
                          _("Turn off the H-DMA emulation. Pressing "
                            "\'0\' during a game toggles H-DMA on "
                            "and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOSPEEDHACKS],
                          _("Turn off a couple of speed hacks. The hacks "
                            "boost the speed of many ROMs "
                            "but cause problems a few ROMs."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->check[NOWINDOWS],
                          _("Disable graphics windows emulation. Use "
                            "\'backspace\' key during a game to "
                            "toggle the emulation on and off."), NULL);
    tool = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tool, rpd->load_defaults,
                          _("Reset this ROM to the defaults specified in "
                            "Global Preferences."), NULL);
	
}

void rom_prefs_dialog_set_status(ROMPrefsData *rpd)
{
    gint i, j;
    gchar *text;
    gint numberops[] = {
        SOUNDSKIP,
        SOUNDQUALITY,
        BUFFERSIZE,
        FRAMESKIP,
        FRAMETIME,
        CYCLES
    };

    text = rpd->rom_info->string_prefs[ROM_FILE];
    if (text && strlen(text)) {
        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(rpd->entry[ROM_FILE]), 
                                      text);
    }
    gtk_entry_set_text(GTK_ENTRY (rpd->entry[NAME]),
                       rpd->rom_info->string_prefs[NAME]);
    gtk_entry_set_text(GTK_ENTRY (rpd->entry[COMPANY]),
                       rpd->rom_info->string_prefs[COMPANY]);
    text = rpd->rom_info->string_prefs[IMAGE_FILE];
    if (text && strlen(text)) {
        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(rpd->entry[IMAGE_FILE]),
                                      text);
    }


    for (i = 0; i < MIX_BOOL_PREFS+ROM_BOOL_PREFS; i++) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rpd->check[i]),
                                     rpd->rom_info->bool_prefs[i]);
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        if (rpd->rom_info->bool_prefs[(numberops[i])]) {
            gtk_spin_button_set_value(GTK_SPIN_BUTTON (rpd->spin[i]),
                                      rpd->rom_info->number_prefs[i]);
        } else {
            gtk_widget_set_sensitive(rpd->spin[i], FALSE);
        }
    }

    if (rpd->rom_info->bool_prefs[NOSOUND]) {
        for (i = SOUNDQUALITY; i < NOMASTERVOLUME; i++) {
            gtk_widget_set_sensitive(rpd->check[i], FALSE);
        }
    } else {
        gtk_widget_set_sensitive(rpd->check[SOUNDSKIP], FALSE);
        gtk_widget_set_sensitive(rpd->spin[SOUNDSKIP_NUMBER], FALSE);
    }

    if (rpd->rom_info->string_prefs[SNAPSHOT_FILE]) {
        gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(rpd->entry[SNAPSHOT_FILE]),
                                      rpd->rom_info->string_prefs[SNAPSHOT_FILE]);
    }
	
    if (!rpd->rom_info->bool_prefs[LOADSNAPSHOT]) {
        gtk_widget_set_sensitive (rpd->entry[SNAPSHOT_FILE], FALSE);
        gtk_widget_set_sensitive (rpd->browse_snapfile, FALSE);
    }
}

void rom_prefs_dialog_get_status(ROMPrefsData *rpd)
{
    const gchar *text;
    gchar *entry;
    gint   i, j, number;	

    for (i = 0; i < ROM_STR_PREFS; i++) {
        if ((i == ROM_FILE) ||
            (i == IMAGE_FILE) ||
            (i == SNAPSHOT_FILE)) {
            text = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(rpd->entry[i]));
        } else { 
            text = gtk_entry_get_text(GTK_ENTRY(rpd->entry[i]));
        }
        if (text && strlen(text)) {
            g_free(rpd->rom_info->string_prefs[i]);
            rpd->rom_info->string_prefs[i] = g_strdup(text);
        }
    }
	
    for (i = 0; i < MIX_BOOL_PREFS+ROM_BOOL_PREFS; i++) {
        rpd->rom_info->bool_prefs[i] = GTK_TOGGLE_BUTTON(rpd->check[i])->active;
    }
    for (i = 0; i < MIX_NUM_PREFS; i++) {
        rpd->rom_info->number_prefs[i] = 
            gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(rpd->spin[i]));
    }
		
    text = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(rpd->entry[SNAPSHOT_FILE]));
	
    if (rpd->rom_info->bool_prefs[LOADSNAPSHOT] && strlen(text)) {
        if (rpd->rom_info->string_prefs[SNAPSHOT_FILE]) {
            g_free(rpd->rom_info->string_prefs[SNAPSHOT_FILE]);
        }
        rpd->rom_info->string_prefs[SNAPSHOT_FILE] = g_strdup(text);
    }
}

void rom_prefs_apply(GtkWidget *widget, ROMPrefsData *rpd)
{
    gint    i, l, new_pos = -1;
    GList   *list;
    gchar   *name, *company, *rom_file, *img_file, *snapshot_file;
    gboolean repeat = FALSE;
    
    if (!rpd->changed) {
        return;
    }
    
    if (!check_rom_info(rpd->entry)) {
        rpd->nodata = TRUE;
        no_rom_file (rpd->rom_prefs_dialog);
        return;
    }

    snapshot_file = 
        gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(rpd->entry[SNAPSHOT_FILE]));
    if ((!snapshot_file || !strlen(snapshot_file)) &&
        GTK_TOGGLE_BUTTON(rpd->check[LOADSNAPSHOT])->active) {
        rpd->nodata = TRUE;
        no_snapfile (rpd->rom_prefs_dialog);
        return;
    }
	
    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(rpd->entry[NAME])));
    company = g_strdup(gtk_entry_get_text(GTK_ENTRY(rpd->entry[COMPANY])));
    rom_file = g_strdup
        (gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(rpd->entry[ROM_FILE])));
    img_file = g_strdup
        (gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(rpd->entry[IMAGE_FILE])));
    
    l = strlen (name);

    for (i = 0; i < l; i++)
        if (name[i] == '/') {
            g_free (name);
            g_free (company);
            g_free (rom_file);
            g_free (img_file);
            no_correct_name (rpd->rom_prefs_dialog);
            return;
        }
    
    if (strcmp (name, rpd->rom_info->string_prefs[NAME])) {
        list = g_list_first (rpd->data->roms_list);
        l    = g_list_length (list);
	
        for (i = 0; i < l; i++) {
            if (!strcmp
                (((ROMInfo*)(list->data))->string_prefs[NAME],
                 name))
                repeat = TRUE;
            list = g_list_next (list);
        }
    }
    
    if (repeat) {
        rom_repeated(name, rpd->rom_prefs_dialog);
        g_free (name);
        g_free (company);
        g_free (rom_file);
        g_free (img_file);
        return;
    }
    
    rom_prefs_dialog_get_status (rpd);
    
    save_rom (rpd->rom_info);
    
    rpd->changed = FALSE;
    
    list = g_list_first (rpd->data->roms_list);
    
    l = g_list_length (list);
    
    gtk_clist_set_text (GTK_CLIST (rpd->data->clist),
                        rpd->data->current_rom, 0, name);
    
    gtk_clist_sort (GTK_CLIST (rpd->data->clist));
    
    g_free (name);
    g_free (company);
    g_free (rom_file);
    g_free (img_file);
    
    for (i = 0; i < l; i++) {
        gtk_clist_get_text (GTK_CLIST (rpd->data->clist),
                            i, 0, &name);
        if (!strcmp (name, rpd->rom_info->string_prefs[NAME]))
            new_pos = i;
    }
    
    rpd->data->current_rom = new_pos;
    
    gtk_clist_select_row (GTK_CLIST (rpd->data->clist),
                          rpd->data->current_rom, 0);
    
}

void rom_repeated(gchar *name, GtkWidget *parent)
{
    GtkWidget *error;

    error = gtk_message_dialog_new(GTK_WINDOW(parent),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
                                   _("Sorry, the ROM \"%s\" "
                                     "already exists."),
                                   name);
    gtk_dialog_run(GTK_DIALOG(error));
    gtk_widget_destroy(error);
}

void no_rom_file(GtkWidget *parent) 
{
    GtkWidget *error;

    error = gtk_message_dialog_new(GTK_WINDOW(parent),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
                                   _("You must fill the \"ROM File\" "
                                     "entry."));
    gtk_dialog_run(GTK_DIALOG(error));
    gtk_widget_destroy(error);
}

void no_snapfile(GtkWidget *parent)
{
    GtkWidget *error;

    error = gtk_message_dialog_new(GTK_WINDOW(parent),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
                                   _("The \"Load snapshot file\" option\n"
                                     "is selected, but the \"Snapshot "
                                     "file\" entry is empty."));
    gtk_dialog_run(GTK_DIALOG(error));
    gtk_widget_destroy(error);
}

void no_correct_name(GtkWidget *parent)
{
    GtkWidget *error;
    
    error = gtk_message_dialog_new(GTK_WINDOW(parent),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_CLOSE,
                                   _("Sorry: the character '/' can't be\n"
                                     "in the name string."));
    gtk_dialog_run(GTK_DIALOG(error));
    gtk_widget_destroy(error);
}

void rom_load_defaults_cb(GtkWidget *widget, ROMPrefsData *rpd)
{
    GlobalPrefs *prefs = g_new(GlobalPrefs, 1);

    prefs = read_conf_file();
    rom_get_defaults(rpd->rom_info, prefs);	
    g_free(prefs);
    rom_prefs_dialog_set_status(rpd);
}


/* romprefs.c ends here. */
