/* 
 * Copyright (C) 1999 Canek Pelez Valds <canek@abulafia.fciencias.unam.mx>
 * Copyright (C) 2007 Manuel Teira Paz <manuel.teira@telefonica.net>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpe-snes.h"
#include <stdio.h>
#include <string.h>


void rom_get_defaults(ROMInfo *rom, GlobalPrefs *prefs)
{
    gint i;
    for (i = 0; i < MIX_BOOL_PREFS; i++) {
        rom->bool_prefs[i] = prefs->bool_prefs[i];
    }

    for (i = 0; i < MIX_NUM_PREFS; i++) {
        rom->number_prefs[i] = prefs->number_prefs[i];
    }
}

GsnesData* gsnes9x_data_new()
{
    GsnesData *data = g_new(GsnesData, 1);

    data->app = NULL;
    data->popmenu = NULL;
    data->rom_widgets = NULL;
    data->paned = NULL;
    data->image_w = NULL;
    data->roms_w = NULL;
    data->pixmap = NULL;
    data->clist = NULL;
    data->status = NULL;
    data->prefs = NULL;
    data->roms_list = NULL;
    data->current_rom = -1;
    data->width = IMAGE_WIDTH;
    data->height = IMAGE_HEIGHT;
	
    return data;
}

GList* get_roms_list()
{
    ROMInfo *rom_info;
    FILE *names;
    GList *list = NULL;
    gchar *roms_file = resolve_dir(ROMS_FILE);
    gchar *str = g_new (gchar, CONF_STR_LEN);
    size_t size = CONF_STR_LEN;
	
    names = fopen(roms_file, "r");
	
    if (!names) {
        return list;
    }

    getline(&str, &size, names);
    str[strlen(str) - 1] = '\0'; /* Rip out the '\n'. */
	
    while (!feof(names)) {
        rom_info = read_rom_info_file(str);

        if (rom_info) {
            if (!rom_info->string_prefs[NAME] ||
                !rom_info->string_prefs[COMPANY] ||
                !rom_info->string_prefs[ROM_FILE]) {
                g_warning(_("The ROM info file %s is corrupt "
                            "or incomplete. Skipping it"),
                          resolve_dir(str));
                g_free(rom_info);
                rom_info = NULL;
            }
        }
        
        if (rom_info)
            list = g_list_prepend(list, (gpointer)rom_info);

        getline(&str, &size, names);
        str[strlen(str) - 1] = '\0';
    }

    fclose(names);
    g_free(str);

    return list;
}

gchar** get_command_line(ROMInfo *rom_info, GlobalPrefs *prefs)
{
    gint i, j, cont = 0;
    gchar **process = g_new (gchar*, 300), *str = g_new (gchar, 100);

    process[cont++] =
        g_strdup (prefs->string_prefs[SNES9X_COMMAND-ROM_STR_PREFS]);
	
    if (rom_info->bool_prefs[TRANSPARENCY])
        process[cont++] = g_strdup ("-transparency");
    if (rom_info->bool_prefs[SIXTEEN])
        process[cont++] = g_strdup ("-sixteen");
    if (rom_info->bool_prefs[HIRES])
        process[cont++] = g_strdup ("-hires");
    if (rom_info->bool_prefs[DISPLAYFRAMERATE])
        process[cont++] = g_strdup ("-displayframerate");
    if (rom_info->bool_prefs[NOSOUND])
        process[cont++] = g_strdup ("-nosound");
    if (rom_info->bool_prefs[SOUNDSKIP]) {
        process[cont++] = g_strdup ("-soundskip");
        g_snprintf (str, 100, "%d", (gint)
                    rom_info->number_prefs[SOUNDSKIP_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[SOUNDQUALITY]) {
        process[cont++] = g_strdup ("-soundquality");
        g_snprintf (str, 100, "%d", (gint)
                    rom_info->number_prefs[SOUNDQUALITY_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[ENVX])
        process[cont++] = g_strdup ("-envx");
    if (rom_info->bool_prefs[SOUNDSYNC])
        process[cont++] = g_strdup ("-soundsync");
    if (rom_info->bool_prefs[INTERPOLATEDSOUND])
        process[cont++] = g_strdup ("-interpolatedsound");
    if (rom_info->bool_prefs[BUFFERSIZE]) {
        process[cont++] = g_strdup ("-buffersize");
        g_snprintf (str, 100, "%d", (gint)
                    rom_info->number_prefs[BUFFERSIZE_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[NOSAMPLECACHING])
        process[cont++] = g_strdup ("-nosamplecaching");
    if (rom_info->bool_prefs[NOECHO])
        process[cont++] = g_strdup ("-noecho");
    if (rom_info->bool_prefs[NOMASTERVOLUME])
        process[cont++] = g_strdup ("-nomastervolume");

    if (rom_info->bool_prefs[FRAMESKIP]) {
        process[cont++] = g_strdup ("-frameskip");
        g_snprintf (str, 100, "%d", (gint)
                    rom_info->number_prefs[FRAMESKIP_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[FRAMETIME]) {
        process[cont++] = g_strdup ("-frametime");
        g_snprintf (str, 100, "%f", (gfloat)
                    rom_info->number_prefs[FRAMETIME_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[CYCLES]) {
        process[cont++] = g_strdup ("-cycles");
        g_snprintf (str, 100, "%d", (gint)
                    rom_info->number_prefs[CYCLES_NUMBER]);
        process[cont++] = g_strdup (str);
    }
    if (rom_info->bool_prefs[INTERLEAVED])
        process[cont++] = g_strdup ("-i");
    if (rom_info->bool_prefs[INTERLEAVED2])
        process[cont++] = g_strdup ("-i2");
    if (rom_info->bool_prefs[HIROM])
        process[cont++] = g_strdup ("-hirom");
    if (rom_info->bool_prefs[LOROM])
        process[cont++] = g_strdup ("-lorom");
    if (rom_info->bool_prefs[HEADER])
        process[cont++] = g_strdup ("-header");
    if (rom_info->bool_prefs[NOHEADER])
        process[cont++] = g_strdup ("-noheader");
    if (rom_info->bool_prefs[PAL])
        process[cont++] = g_strdup ("-pal");
    if (rom_info->bool_prefs[NTSC])
        process[cont++] = g_strdup ("-ntsc");
    if (rom_info->bool_prefs[LAYERING])
        process[cont++] = g_strdup ("-layering");
    if (rom_info->bool_prefs[LOADSNAPSHOT] &&
        rom_info->string_prefs[SNAPSHOT_FILE]) {
        process[cont++] = g_strdup ("-loadsnapshot");
        process[cont++] = g_strdup
            (rom_info->string_prefs[SNAPSHOT_FILE]);
    }
    if (rom_info->bool_prefs[NOHDMA])
        process[cont++] = g_strdup ("-nohdma");
    if (rom_info->bool_prefs[NOSPEEDHACKS])
        process[cont++] = g_strdup ("-nospeedhacks");
    if (rom_info->bool_prefs[NOWINDOWS])
        process[cont++] = g_strdup ("-nowindows");
    process[cont++] = g_strdup (rom_info->string_prefs[ROM_FILE]);
	
    g_free (str);
	
    for (i = cont; i < 300; i++)
        process[i] = NULL;
	
    return process;
}

const gchar* get_default_roms_dir()
{
    GlobalPrefs *prefs = g_new (GlobalPrefs, 1);
    const gchar *path;
	
    prefs = read_conf_file ();	
    path = prefs->string_prefs[ROMS_DIR-ROM_STR_PREFS];
    g_free (prefs);

    /* Try every posible path for ROMS. */
    if (!path) {
        path = g_getenv ("GSNES9X_ROM_DIR");
        if (!path)
            path = g_getenv ("SNES96_ROM_DIR");
        if (!path)
            path = g_get_home_dir ();
        if (path)
            path = g_strconcat (path, "/", NULL);
    }
    return path;
}

const gchar* get_default_images_dir()
{
    const gchar *path;

    /* Try every posible path for images. */
    path = g_getenv("GSNES9X_IMAGE_DIR");
    if (!path) {
        path = g_get_home_dir();
    }
    if (path) {
        path = g_strconcat(path, "/", NULL);
    }
    return path;
}
 
const gchar* get_default_snap_dir ()
{
    const gchar *path;

    /* Try every posible path for snaps. */
    path = g_getenv("GSNES9X_SNAPSHOT_DIR");
    if (!path) {
        path = g_getenv("SNES96_SNAPSHOT_DIR");
    }
    if (!path) {
        path = g_strconcat(g_get_home_dir(), "/.snes96_snapshots", NULL);
    }
    if (path) {
        path = g_strconcat(path, "/", NULL);
    }
    return path;
}

gboolean check_rom_info(GtkWidget **entry)
{
    gchar      **sides;
    gchar      *temp_name;
		
    F_NAME();
	
    if (!strlen(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(entry[ROM_FILE])))) {
        return FALSE;
    }

    /* If name is empty, get name from rom file name. */	
    if (!strlen(gtk_entry_get_text(GTK_ENTRY(entry[NAME])))) {
        sides = g_strsplit(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(entry[ROM_FILE])), ".", 2);
        temp_name = strrchr(sides[0], '/');
        if (temp_name == NULL) {
            gtk_entry_set_text(GTK_ENTRY(entry[NAME]),
                               g_strdup(sides[0]));
        } else {
            gtk_entry_set_text(GTK_ENTRY(entry[NAME]),
                               g_strdup (++temp_name));
        }
        g_strfreev (sides);
    }

    /* If company or image file is empty, put something in there. */
    if (!strlen(gtk_entry_get_text(GTK_ENTRY(entry[COMPANY]))))
        gtk_entry_set_text (GTK_ENTRY (entry[COMPANY]), "?");
        
    return TRUE;	
}

gchar *resolve_dir(gchar *dirname)
{
    static gchar *basedir = NULL;
    gchar *result = NULL;
    
    if (basedir == NULL) {
        basedir = g_strdup_printf("%s/.gpe/", g_get_home_dir());
    }
    result = g_strconcat(basedir, dirname, NULL);
    //g_free(dirname);
    return result;
}

GtkWidget* get_image_pixmap(gchar *file, gint width, gint height)
{
    GdkPixbuf *pixbuf = NULL;
	
    if (file) {
        pixbuf = gdk_pixbuf_new_from_file_at_size(file, width, height, NULL);
    }

    if (!pixbuf) {
        pixbuf = gdk_pixbuf_new_from_file_at_size(DEFAULT_IMAGE, 
                                                  width, height, NULL);
    }
    if (pixbuf) {
        return gtk_image_new_from_pixbuf(pixbuf);
    } else {
        return NULL;
    }
}
